"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const cli_ux_1 = require("cli-ux");
const fs_1 = require("fs");
const path_1 = require("path");
const marketplace_command_1 = tslib_1.__importDefault(require("../../marketplace-command"));
const services_1 = tslib_1.__importDefault(require("../../services"));
const deploy_1 = tslib_1.__importDefault(require("../service/deploy"));
const detail_1 = tslib_1.__importDefault(require("../service/detail"));
const ipfsClient = require('ipfs-http-client');
class MarketplacePublish extends marketplace_command_1.default {
    constructor() {
        super(...arguments);
        this.IPFS = ipfsClient('ipfs.app.mesg.com', '5001', { protocol: 'http' });
    }
    async run() {
        const { args } = this.parse(MarketplacePublish);
        const path = args.SERVICE_PATH;
        const account = await this.getAccount();
        this.spinner.start('Deploy service');
        const [deployedService] = (await deploy_1.default.run([path, '--silent']));
        this.require(deployedService, 'Deployed service issue');
        const [service] = (await detail_1.default.run([deployedService.hash, '--silent']));
        this.spinner.status = 'Upload sources';
        const sources = await this.deploySources(path);
        this.spinner.status = 'Preparing service';
        const serviceTx = await this.preparePublishService({
            definition: service.definition,
            readme: this.lookupReadme(path),
            hash: service.definition.hash,
            hashVersion: '1',
            deployment: {
                type: 'ipfs',
                source: sources
            }
        }, account);
        this.spinner.stop('ready');
        if (!await cli_ux_1.cli.confirm(`Ready to send a transaction to ${serviceTx.to} with the account ${account}?`)) {
            return null;
        }
        const passphrase = await this.getPassphrase();
        this.spinner.start('Publish service');
        const marketplaceService = await this.publishService(account, serviceTx, passphrase);
        this.styledJSON(marketplaceService);
        return marketplaceService;
    }
    async deploySources(path) {
        const cmd = new deploy_1.default(this.argv, this.config);
        const buffer = [];
        return new Promise((resolve, reject) => {
            cmd.createTar(path_1.join(path))
                .on('data', (data) => buffer.push(...data))
                .once('error', reject)
                .on('end', async () => resolve(await this.upload(Buffer.from(buffer))));
        });
    }
    async preparePublishService(service, account) {
        const publishTx = await this.executeAndCaptureError(services_1.default.marketplace.id, services_1.default.marketplace.tasks.prepareCreateVersion, {
            service,
            from: account
        });
        return publishTx.data;
    }
    async publishService(account, serviceTx, passphrase) {
        const signedTx = await this.sign(account, serviceTx, passphrase);
        const service = await this.executeAndCaptureError(services_1.default.marketplace.id, services_1.default.marketplace.tasks.publishCreateVersion, signedTx);
        return service.data;
    }
    lookupReadme(path) {
        const readmeRegexp = /^readme(.md)?$/i;
        const readme = fs_1.readdirSync(path).find(x => !!x.match(readmeRegexp));
        if (!readme) {
            return '';
        }
        return fs_1.readFileSync(path_1.join(path, readme)).toString();
    }
    async upload(buffer) {
        const res = await this.IPFS.add(Buffer.from(buffer), { pin: true });
        if (!res.length) {
            throw new Error('Error with the generation of your manifest');
        }
        return res[0].hash;
    }
}
MarketplacePublish.description = 'Publish a service on the MESG Marketplace';
MarketplacePublish.args = [{
        name: 'SERVICE_PATH',
        description: 'Path of the service',
        required: true,
        default: './'
    }];
exports.default = MarketplacePublish;
